# frozen_string_literal: true
require 'gosu'
require 'menuitem'
require 'game'

module ZOrder
  BACKGROUND, STARS, PLAYER, UI = *0..3
end

class Scene
  attr_reader :window

  def initialize(window)
    @window = window
  end

  def update; end

  def draw; end

  def button_down(id) end
end

class MainMenuScene < Scene

  def initialize(window)
    super(window)

    @continue = Continue.new(150, 50, window)
    @new_game = NewGame.new(150, 100, window)
    @difficulty = Difficulty.new(150, 150, window)
    @high_score = HighScore.new(150, 200, window)
    @quit = Quit.new(150, 250, window)

    @menu_items = [@new_game,
                   @difficulty,
                   @high_score,
                   @quit]
    @selected_item = 0
    @menu_items[0].select
  end

  def difficulty
    @difficulty.difficulty
  end

  def add_continue
    if @menu_items.size == 5
      @menu_items[@selected_item].unselect
      @selected_item = 0
    else
      @menu_items[0].unselect
      @menu_items.prepend(@continue)
    end

    @menu_items[0].select
  end

  def remove_continue
    @menu_items[0].unselect
    @menu_items.drop(1)
    @menu_items[0].select
    @selected_item = 0
  end

  def button_down(id)
    case id
    when Gosu::KB_DOWN
      down
    when Gosu::KB_UP
      up
    else
      @menu_items[@selected_item].button_down(id)
    end
  end

  def draw
    @menu_items.each(&:draw)
  end

  def down
    @menu_items[@selected_item].unselect

    @selected_item = if @selected_item == @menu_items.size - 1
                       0
                     else
                       @selected_item + 1
                     end

    @menu_items[@selected_item].select
  end

  def up
    @menu_items[@selected_item].unselect

    @selected_item = if @selected_item.zero?
                       @menu_items.size - 1
                     else
                       @selected_item - 1
                     end

    @menu_items[@selected_item].select
  end
end

class GameScene < Scene
  WIDTH = 10
  HEIGHT = 20
  BLOCK_SIZE = 20 # pixels

  def initialize(window, difficulty)
    super(window)
    @game = Game.new(window,10, 50, HEIGHT, WIDTH, BLOCK_SIZE, difficulty)
    @ticks = 0
  end

  def update
    @game.update
  end

  def button_down(id)

    case id
    when Gosu::KB_ESCAPE
      @window.menu
    when Gosu::KB_LEFT
      @game.move(-1, 0)
    when Gosu::KB_RIGHT
      @game.move(1, 0)
    when Gosu::KB_UP
      @game.rotate
    when Gosu::KB_DOWN
      @game.move(0, 1)
    when Gosu::KB_SPACE
      @game.fast_move
    end
  end

  def game_running?
    !@game.game_over
  end

  def draw
    @window.font.draw_text("Score: #{@game.score}", 10, 10, ZOrder::UI, 1.0, 1.0, Gosu::Color::YELLOW)
    @window.font.draw_text("Speed: #{@game.current_difficulty}", 250, 10, ZOrder::UI, 1.0, 1.0, Gosu::Color::YELLOW)
    @game.draw(@window)
  end

end

class HighScoreScene < Scene
  WIDTH = 10
  HEIGHT = 20
  BLOCK_SIZE = 20 # pixels

  def initialize(window)
    super(window)
    @list = [0]

  end

  def add_score(score)
    @list.append(score)
    @list = @list.sort.reverse
  end

  def button_down(id)
    @window.menu if id == Gosu::KB_ESCAPE
  end

  def draw

    i = 1
    x = 150
    y = 10
    @list.each do |l|
      @window.font.draw_text("#{i}. #{l}", x, y, ZOrder::UI, 1.0, 1.0, Gosu::Color::YELLOW)
      i += 1
      y += 50
      return if i >= 10
    end

  end

end
